from gru.agents.framework_wrappers import BaseAgent
from langgraph.graph import StateGraph

from gru.agents.framework_wrappers.langgraph.workflow import LanggraphWorkflow
from gru.agents.service.app import App

class CansoLanggraphAgent(BaseAgent):

    def __init__(self, stateGraph: StateGraph) -> None:
        workflow = LanggraphWorkflow(stateGraph)
        self.app = App(workflow)

    def run(self):
        self.app.run()




import json
import os
from typing import Any

import psycopg
from psycopg_pool import AsyncConnectionPool
from gru.agents.checkpoint.postgres import PostgresAsyncConnectionPool
from gru.agents.framework_wrappers import AgentWorkflow
from langgraph.graph import StateGraph

from gru.agents.schemas import AgentInvokeRequest
from langchain_core.runnables import RunnableConfig
from langgraph.checkpoint.postgres.aio import AsyncPostgresSaver

from gru.agents.schemas.schemas import TaskCompleteRequest
from langchain_core.messages import ToolMessage

class LanggraphWorkflow(AgentWorkflow):
    
    def __init__(self, stateGraph: StateGraph) -> None:
        super().__init__()
        self.state_graph = stateGraph

    async def setup(self):
        checkpoint_db_type = os.getenv("CHECKPOINT_DB_TYPE", "postgres")
        if checkpoint_db_type == "postgres":
            pool = PostgresAsyncConnectionPool().get()
            checkpointer = await self._setup_postgres_checkpointer(pool)
            self.compiled_graph = self.state_graph.compile(checkpointer)


    async def invoke(self, request: AgentInvokeRequest) -> dict[str, Any]:
        config = RunnableConfig(
            configurable={"thread_id": request.prompt_id},
        )
        return await self.compiled_graph.ainvoke(input=request.prompt_body, config=config)
    
    async def resume(self, request: TaskCompleteRequest) -> dict[str, Any]:
        config = RunnableConfig(
            configurable={"thread_id": request.prompt_id},
        )
        state = await self.compiled_graph.aget_state(config=config)

        ## todo : Support for Custom state?
        await self.compiled_graph.aupdate_state(
            config=config, 
            values={"messages": [ToolMessage(content=json.dumps(request.result), name=request.task_type, tool_call_id=request.tool_call_id)]}, 
            as_node=state.next[0]
        )

        result = await self.compiled_graph.ainvoke(input=None, config=config)
        return result
    
    async def _setup_postgres_checkpointer(self, pool: AsyncConnectionPool):
        checkpointer = AsyncPostgresSaver(pool)
        async with pool.connection() as conn:
            async with conn.cursor() as cur:
                try:
                    await cur.execute("""
                        SELECT EXISTS (
                            SELECT FROM information_schema.tables 
                            WHERE  table_schema = 'public'
                            AND    table_name   = 'checkpoints'
                        );
                    """)
                    table_exists = (await cur.fetchone())[0]
                    
                    if not table_exists:
                        print("Checkpoints table does not exist. Running setup...")
                        await checkpointer.setup()
                    else:
                        print("Checkpoints table already exists. Skipping setup.")
                except psycopg.Error as e:
                    print(f"Error checking for checkpoints table: {e}")
                    raise e
        return checkpointer
    

    



from typing import Any
from pydantic import BaseModel


class AgentInvokeRequest(BaseModel):
    prompt_id : str
    prompt_body : dict[str, Any]

class AgentInvokeResponse(BaseModel):
    prompt_id : str

class TaskCompleteRequest(BaseModel):
    prompt_id:str
    tool_call_id: str
    task_type: str
    status: str
    result: dict[str, Any]


from abc import ABC, abstractmethod
from typing import Any

from gru.agents.schemas import AgentInvokeRequest
from gru.agents.schemas.schemas import TaskCompleteRequest


class AgentWorkflow(ABC):

    @abstractmethod
    async def setup(self):
        pass
    
    @abstractmethod
    async def invoke(self, request: AgentInvokeRequest) -> dict[str, Any]:
        pass

    @abstractmethod
    async def resume(self, request: TaskCompleteRequest) -> dict[str, Any]:
        pass


from pydantic import BaseModel
from typing import Dict, Any

class AgentPromptRequest(BaseModel):
    """
    Model representing a prompt request to an agent.
    
    Attributes:
        prompt (Dict[str, Any]): Dictionary containing the prompt data
    """
    prompt: Dict[str, Any]

class AgentRegisterRequest(BaseModel):
	cluster_name: str
	agent_name: str
	image: str
	image_pull_secret: str
	task_server_name: str
	checkpoint_db_name: str
	replicas: int



import os
from typing import Any
from celery import Celery
from pydantic import BaseModel

broker = os.getenv("BROKER_URL")

task_server = Celery('task_server', broker=broker)

class TaskMetadata(BaseModel):
    prompt_id: str
    tool_call_id: str
    agent_name: str
    agent_callback_url: str

def submit_task(task_type: str, metadata: TaskMetadata, task_attributes: dict[str, Any]):
    task_metadata = metadata.model_dump()
    task_server.send_task("src.tasks.execute", args=[task_type, task_metadata, task_attributes])


from contextlib import asynccontextmanager
from fastapi import BackgroundTasks, FastAPI, Response
import uvicorn
from gru.agents.framework_wrappers import AgentWorkflow
from gru.agents.schemas import AgentInvokeRequest, AgentInvokeResponse
from gru.agents.schemas.schemas import TaskCompleteRequest

@asynccontextmanager
async def lifespan(app: FastAPI):

    workflow: AgentWorkflow = app.state.workflow
    await workflow.setup()
    yield
        
api = FastAPI(lifespan=lifespan)

async def invoke_workflow(request: AgentInvokeRequest):
    workflow: AgentWorkflow = api.state.workflow
    output = await workflow.invoke(request)
    # Todo: Save output to DB table
    print(output)

async def resume_workflow(request: TaskCompleteRequest):
    workflow: AgentWorkflow = api.state.workflow
    output = await workflow.resume(request)
    # Todo: Save output to DB table
    print(output)

@api.post("/invoke")
async def invoke(request: AgentInvokeRequest, background_tasks: BackgroundTasks) -> AgentInvokeResponse:
    background_tasks.add_task(invoke_workflow, request)
    return AgentInvokeResponse(prompt_id=request.prompt_id)

@api.post("/task-complete")
async def task_complete(request: TaskCompleteRequest, background_tasks: BackgroundTasks):
    background_tasks.add_task(resume_workflow, request)
    return Response(status_code=200)

class App:

    def __init__(self, workflow: AgentWorkflow):
        api.state.workflow = workflow

    def run(self):
        uvicorn.run(api, host="0.0.0.0", port=8080)


import json
import os
import yaml

from gru import DEFAULT_CANSO_API_CONFIG
from gru.agents.models import AgentPromptRequest, AgentRegisterRequest
from gru.schema.api_request_handler import APIRequestHandler
from gru.schema.api_response_handler import APIResponseHandler
from gru.utils.config_reader import ConfigReader

from cookiecutter.main import cookiecutter

AGENT_CONFIG_FILE_NAME = "config.yaml"

AI_AGENT_TEMPLATE_URL = "git@github.com:Yugen-ai/canso-ai-agent-templates.git"

def ai_agent_templates_setup():
    """
    Run Cookiecutter with the specified template.
    """
    cookiecutter(AI_AGENT_TEMPLATE_URL, checkout="agent-template-changes")

def register_agent(auth_token, agent_folder, cluster_name, image, image_pull_secret):
    
    config_file_path = os.path.join(agent_folder, AGENT_CONFIG_FILE_NAME)

    with open(config_file_path, "r") as config_file:
            config_dict = yaml.safe_load(config_file)
    
    agent_register_request = AgentRegisterRequest(
          cluster_name=cluster_name,
          agent_name=config_dict["agent_name"],
          image=image,
          image_pull_secret=image_pull_secret,
          task_server_name=config_dict["task_server_name"],
          checkpoint_db_name=config_dict["checkpoint_db_name"],
          replicas=config_dict["replicas"]
    )

    request_json = agent_register_request.model_dump()

    configs = ConfigReader(DEFAULT_CANSO_API_CONFIG)
    request_handler = APIRequestHandler(auth_token, configs)
    response = request_handler.send_request("post", configs.agent_register_endpoint, request_json)
    response_handler = APIResponseHandler(response)
    response_handler.check_for_errors()

    return response_handler.get_message()

def deploy_agent(auth_token, agent_name):

    configs = ConfigReader(DEFAULT_CANSO_API_CONFIG)
    request_handler = APIRequestHandler(auth_token, configs)
    url = configs.agent_deploy_endpoint.replace("{agent_name}", agent_name)
    response = request_handler.send_request("post", url)
    response_handler = APIResponseHandler(response)
    response_handler.check_for_errors()

    return response_handler.get_message()

def prompt_agent(agent_name: str, prompt: dict, auth_token: str):
    """
    Send a prompt to a deployed agent.
    
    Args:
        agent_name (str): Name of the deployed agent
        prompt (dict): Dictionary containing the prompt data
        auth_token (str): Authentication token for API access
    
    Returns:
        AgentPromptResponse: Response containing the prompt ID
        
    Raises:
        ApiError: If the API request fails
        ValueError: If the request parameters are invalid
    """
    prompt_request = AgentPromptRequest(prompt=prompt)
    request_json = prompt_request.model_dump()
    
    configs = ConfigReader(DEFAULT_CANSO_API_CONFIG)
    request_handler = APIRequestHandler(auth_token, configs)
    
    endpoint = configs.agent_prompt_endpoint.replace("{agent_name}", agent_name)
    
    response = request_handler.send_request("post", endpoint, request_json)
    response_handler = APIResponseHandler(response)
    response_handler.check_for_errors()
    
    response_data = response_handler.get_success_data()
    return response_data

def read_prompt_file(prompt_file: str) -> dict:
    """
    Read and parse a JSON prompt file.
    
    Args:
        prompt_file (str): Path to the JSON file containing prompt data
        
    Returns:
        dict: Parsed prompt data
        
    Raises:
        FileNotFoundError: If the file doesn't exist
        json.JSONDecodeError: If the file contains invalid JSON
    """
    try:
        with open(prompt_file, "r") as f:
            return json.load(f)
    except FileNotFoundError:
        raise FileNotFoundError(f"Prompt file not found: {prompt_file}")
    except json.JSONDecodeError as e:
        raise json.JSONDecodeError(
            f"Invalid JSON in {prompt_file}: {str(e)}", 
            e.doc, 
            e.pos
        )




import setuptools

setuptools.setup(
    name='gru',
    version='0.0.1rc20',
    install_requires=['requests>=2.31.0', 'typing==3.7.4.3','chardet==5.1.0','click>=8.1.7', 'fire>=0.7.0', 'pydantic>=2.6.3', 'cookiecutter>=2.6.0', 'langgraph>=0.2.50', 'fastapi>=0.115.5', 'uvicorn>=0.32.0', 'langgraph-checkpoint-postgres>=2.0.3', 'psycopg[binary,pool]', 'celery>=5.4.0', 'redis>=5.2.0'],
    entry_points={
        'console_scripts': [
            'yugenml = gru.cookiecutter.mlops_templates_cli:mlops_template_cli',
            'yserve = gru.ml_serving.server:serve',
            'gru = gru.cli:main',
            'canso = gru.canso_cli:main'
        ],
    },
    packages=setuptools.find_packages(),
    )